/*
   xdm_gfx.cc
*/

#include <stdio.h>

#include "const.h"
#include "pieces16.xpm"
#include "title.xpm"

#include "xdm_gfx.h"


DynaPixels::DynaPixels( int x, int y)
: QPixmap( x, y)
{
    title = new QPixmap( title_xpm);
    pieces16 = new QPixmap( pieces16_xpm);
    
    sprintf( text, "Right click to select a level to play.");
    
    drawTitle();
}

void DynaPixels::drawTitle()
{
    QPainter  paint( this );
    
    
    posx = -1;
    
    fill( QColor( 168, 168, 168));
    bitBlt( this, 0, 0, title);
    
    paint.setPen( black);
    paint.drawText( rect(), AlignCenter, text);
    
    if ( dynscr != 0L ) bitBlt( dynscr, 0, 0, this);
}

void DynaPixels::drawLevel()
{
    int  x, y, ofs;
    
    
    posx = 0;
    dirx = 0;
    diry = 0;
    
    for ( y = 0; y < 16; y++ ) for ( x = 0; x < 16; x++ )
    {
        ofs = lookuptable[ leveldata[ x ][ y ] ] * 16;
        bitBlt( this, x * 16, y * 16, pieces16, 0, ofs, 16, 16);
    }
    
    bitBlt( dynscr, 0, 0, this);
}

void DynaPixels::death()
{
    char  sl[ 64 ];
    FILE  *level;
    
    
    // dont die if explosion enters death
    if ( leveldata[ posx ][ posy ] == S_EMPTY && stonesleft ) return;
    
    dirx = 0; diry = 0;
    
    if ( stonesleft )
    {
        if ( stonesleft == 1 ) sprintf( sl, ",\n with 1 stone left.");
        else sprintf( sl, ",\n with %i stones left.", stonesleft);
        if ( moves == 1 ) sprintf( text, "Abandoned after 1 move%s", sl);
        else sprintf( text, "Abandoned after %i moves%s", moves, sl);
    }
    else
    {
        if ( moves > best ) sprintf( sl, "\nThe record is %i.", best);
        else if ( moves < best )
        {
            sprintf( sl, "\nA new record! The old was %i.", best);
            level = fopen( curlevel, "rb+");
            if ( level != 0L )
            {
                fseek( level, 256, SEEK_SET);
                fputc( moves, level);
                fclose( level);
            }
        }
        else sprintf( sl, "\nThat is the old record.");
        
        if ( moves == 1 ) sprintf( text, "Completed in 1 move.%s", sl);
        else sprintf( text, "Completed in %i moves.%s", moves, sl);
    }
    
    drawTitle();
}

void DynaPixels::explode( int x, int y, int type)
{
    leveldata[ x ][ y ] = S_EMPTY;
    bitBlt( this, x * 16, y * 16, pieces16, 0, 0, 16, 16);
    bitBlt( dynscr, x * 16, y * 16, pieces16, 0, 0, 16, 16);
    stonesleft--;
    
    if ( leveldata[ posx ][ posy ] != S_EMPTY )
    {
        leveldata[ posx ][ posy ] = S_EMPTY;
        bitBlt( this, posx * 16, posy * 16, pieces16, 0, 0, 16, 16);
        bitBlt( dynscr, posx * 16, posy * 16, pieces16, 0, 0, 16, 16);
        stonesleft--;
    }
    
    if ( stonesleft == 0 ) death();
    
    getnext( x, y, type);
    
    dirx = 0; diry = 0;
}

void DynaPixels::getnext( int &x, int &y, int &type)
{
    int  i, j, ctype;
    
    
Again:
        x += dirx;
        y += diry;
        
        if ( x < 0 || y < 0 || x > 15 || y > 15 || moves > 250 )
        {
            death();
            return;
        }
        
        ctype = leveldata[ x ][ y ] & 127;
        switch ( ctype )
        {
            case S_EMPTY: break;
            case S_PASS_HORI:
                if ( diry != 0 ) { diry = 0; break; }
                else goto Again;
            case S_PASS_VERT:
                if ( dirx != 0 ) { dirx = 0; break; }
                else goto Again;
            case S_CYKLECOL:
                if ( type >= 0xF8 && type <= 0xFA )
                {
                    type++;
                    if ( type > 0xFA ) type = 0xF8;
                }
                goto Again; break;
            case S_RED:
                if ( type == M_BLACK || type == M_BLUE ) explode( x, y, type);
                dirx = 0; diry = 0; break;
            case S_GREEN:
                if ( type == M_BLACK || type == M_RED ) explode( x, y, type);
                dirx = 0; diry = 0; break;
            case S_BLUE:
                if ( type == M_BLACK || type == M_GREEN ) explode( x, y, type);
                dirx = 0; diry = 0; break;
            case S_BLACK:
                if ( type == M_BLACK ) explode( x, y, type);
                dirx = 0; diry = 0; break;
            case S_YELLOW:
                if ( (type & 0xF0) == 0xF0 && type != M_YELLOW ) explode( x, y, type);
                dirx = 0; diry = 0; break;
            case S_CYKLEDIR_CW: switch ( dirx ) {
                case 1: dirx = 0; diry = 1; break;
                case -1: dirx = 0; diry = -1; break;
                default: switch ( diry ) {
                case 1: dirx = -1; diry = 0; break;
                default: dirx = 1; diry = 0; break; } }
                goto Again; break;
            case S_CYKLEDIR_CCW: switch ( dirx ) {
                case 1: dirx = 0; diry = -1; break;
                case -1: dirx = 0; diry = 1; break;
                default: switch ( diry ) {
                case 1: dirx = 1; diry = 0; break;
                default: dirx = -1; diry = 0; break; } }
                goto Again; break;
            case S_TELEP_0:
            case S_TELEP_1:
                if ( type != ctype && type != (ctype + 128) )
                {
                    for ( j = 0; j < 16; j++ ) for ( i = 0; i < 16; i++)
                    {
                        if ( (leveldata[ i ][ j ] & 127) == ctype &&
                            x != i && y != j )
                        {
                            x = i; y = j;
                            i = 16; j = 16;
                        }
                    }
                }
                goto Again; break;
            case S_DEATH: death(); break;
            case S_GLUE0:
                gluesleft = 1;
                /*explode?*/
                break;
            case S_GLUE3:
                gluesleft = 3;
                /*explode?*/
                break;
            case S_GLUE5:
                gluesleft = 5;
                /*explode?*/
                break;
            default: dirx = 0; diry = 0; break;
        }
}

void DynaPixels::timer()
{
    int  x, y, ofs, type;
    
    
    if ( dirx != 0 || diry != 0 )
    {
        x = posx;
        y = posy;
        type = leveldata[ posx ][ posy ];
        
        getnext( x, y, type);
        
        if ( dirx != 0 || diry != 0 )
        {
            leveldata[ x ][ y ] = type;
            bitBlt( this, posx * 16, posy * 16, pieces16, 0, 0, 16, 16);
            bitBlt( dynscr, posx * 16, posy * 16, pieces16, 0, 0, 16, 16);
            
            leveldata[ posx ][ posy ] = S_EMPTY;
            ofs = lookuptable[ leveldata[ x ][ y ] ] * 16;
            bitBlt( this, x * 16, y * 16, pieces16, 0, ofs, 16, 16);
            bitBlt( dynscr, x * 16, y * 16, pieces16, 0, ofs, 16, 16);
            
            posx = x;
            posy = y;
            
            if ( gluesleft )
            {
                gluesleft--;
                dirx = 0; diry = 0;
            }
        }
    }
}
